/* ======================================================================== */
/* allocate.cpp                                                             */
/*                                                                          */
/*       This subroutine allocates memory by using the calloc command on    */
/* structures found in defines.h.                                           */
/*                                                                          */
/* ======================================================================== */

#include "defines.h"


/* ************************************************************************ */
/*                                                                          */
/* ======================== Begin Subroutine ============================== */
/*                                                                          */
/* ************************************************************************ */

void allocm (void) 
{
	/* ================================================================== */
	/* Allocate memory for all sites.                                     */
	/*                                                                    */
	/*	Here, the sturcture called "atom" has two indices.  The first   */
	/* is the box number and the second is the site number.               */
	/*                                                                    */
	/* ================================================================== */
	atom    = (struct atoms**) calloc(sim.NB,sizeof(struct atoms*));
		if(atom == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for atom\n"); exit(11);}
	nlist = (struct nlist_def*) calloc(sim.NB,sizeof(struct nlist_def));
		if(nlist == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for nlist\n"); exit(11);}

	for(int i=0; i<sim.NB; i++) {
		int na = box[i].boxns + MEXTRA;
		atom[i]    = (struct atoms*) calloc(na,sizeof(struct atoms));
			if(atom[i] == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for atom[i]\n"); exit(11);}
	}


	/* ================================================================== */
	/* Allocate memory for the residue structure.                         */
	/*                                                                    */
	/*	Here, the sturcture called "residue" has two indices.  The      */
	/* first is the box number and the second is the residue number.      */
	/*                                                                    */
	/* ================================================================== */
	residue = (struct res**) calloc(sim.NB,sizeof(struct res*));
		if(residue == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for residue"); exit(11);}
	for(int i=0; i<sim.NB; i++) {
		int na = box[i].boxnres + MEXTRA;						// boxnres = total number of amino acid residue in box i
		residue[i] = (struct res*) calloc(na,sizeof(struct res));
			if(residue[i] == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for residue[i]\n"); exit(11);}
	}

	/* ================================================================== */
	/* Allocate memory for the force structures.                          */
	/*                                                                    */
	/*	Here, the sturctures called "ff", "ff_short", & ff_long" have   */
	/* two indices.  The first is the box number and the second is the    */
	/* site/atom number.                                                  */
	/* ================================================================== */
	ff = (struct veloc**) calloc(sim.NB,sizeof(struct veloc*));
		if(ff == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for ff\n"); exit(11);}
	for(int i=0; i<sim.NB; i++) {
		int fn = box[i].boxns + MEXTRA;
		ff[i] = (struct veloc*) calloc(fn,sizeof(struct veloc));
			if(ff[i] == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for ff[i]\n"); exit(11);}
	}

	/* ================================================================== */
	/*                                                                    */
	/* Allocate memory for the force field parameter.                     */
	/*                                                                    */
	/* ================================================================== */
	pott  = (struct inter**) calloc(sim.NB,sizeof(struct inter*));
		if(pott == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for pott\n"); exit(11);}

	bond  = (struct bonds**) calloc(sim.NB,sizeof(struct bonds*));
		if(bond == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for bond\n"); exit(11);}
	bend  = (struct bends**) calloc(sim.NB,sizeof(struct bends*));
		if(bend == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for bend\n"); exit(11);}
	tors  = (struct torsions**) calloc(sim.NB,sizeof(struct torsions*));
		if(tors == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for tors\n"); exit(11);}
	xint  = (struct excinter**) calloc(sim.NB,sizeof(struct excinter*));
		if(xint == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for xint\n"); exit(11);} 
	table = (struct hashlist**) calloc(sim.NB,sizeof(struct hashlist*));
		if(table == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for table\n"); exit(11);}


	/* ================================================================== */
	/* Allocate memory for the neighbor list structure.                   */
	/*                                                                    */
	/*	Here, the sturcture called "nlist" has one indice for the       */
	/* box number.  The "count" member of the structure has one indice    */
	/* for the site/atom number and the member "list" has two indices.    */
	/* The first is the site/atom number and the second is the site       */
	/* number of its neighbor.                                            */
	/*                                                                    */
	/* ================================================================== */
	for(int k=0; k<sim.NB; k++) {

		int size = box[k].boxns;

			if(size < 2000) max_num_neighbors = size;
		else max_num_neighbors = 1500;


		max_num_neighbors = 1000;


		nlist[k].count = (int*) calloc(size,sizeof(int));
			if(nlist[k].count == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for nlist[k].count\n"); exit(11);}
		nlist[k].list  = (int**) calloc(size,sizeof(int*));
			if(nlist[k].list == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for nlist[k].list\n"); exit(11);}

		nlist[k].xflag = (int**) calloc(size,sizeof(int*));
			if(nlist[k].xflag == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for nlist[k].xflag\n"); exit(11);}


		nlist[k].sig = (double**) calloc(size,sizeof(double*));
			if(nlist[k].sig == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for nlist[k].sig\n"); exit(11);}

		for(int i=0; i<size; i++){ 
			nlist[k].list[i]   = (int*) calloc(max_num_neighbors,sizeof(int));
				if(nlist[k].list[i] == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for nlist[k].list[i]\n"); exit(11);}

			nlist[k].xflag[i]  = (int*) calloc(max_num_neighbors,sizeof(int));
				if(nlist[k].xflag[i] == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for nlist[k].xflag[i]\n"); exit(11);}


			nlist[k].sig[i]  = (double*) calloc(max_num_neighbors,sizeof(double));
				if(nlist[k].sig[i] == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for nlist[k].sig[i]\n"); exit(11);}

		}

	}

}
