/* ======================================================================== */
/* forces.cpp                                                               */
/*                                                                          */
/*		This subroutine is the driver for the force calculations if the     */
/* multiple time step (RESPA) integration is not selected in simul.input.   */
/* It calls the subroutines that calculate the different contributions to   */
/* the forces.  It zero's out the forces on the atoms (e.g. ff[k][i].x) so  */
/* that the contributions to the different energy modes can be added in     */
/* their respective subroutines (e.g. cbond()).  It also zero's out the     */
/* virial accumulators.                                                     */
/*                                                                          */
/*                                                                          */
/* Passed Parameters:                                                       */
/*						ibox:		The box number                          */
/*                                                                          */
/* ======================================================================== */

#include "defines.h"

/* ================================================================== */
/*                                                                    */
/* Function Prototypes                                                */
/*                                                                    */
/* ================================================================== */
double cbond (int);
double cbend (int);
double ctorsion (int);
double cimproper (int);
void   cnonbondnl (int,double*);
double cbb_golik(int);
double cba_golik(int);
double cda_golik(int);
void cnbnd_dnagolik(int,double*);

/* ************************************************************************ */
/*                                                                          */
/* ======================== Begin Subroutine ============================== */
/*                                                                          */
/* ************************************************************************ */

void forces_format (int ibox)
{
	int k = ibox;
	/* ================================================================== */
	/*                                                                    */
	/* Zero out atomic forces arrays.                                     */
	/*                                                                    */
	/* ================================================================== */
	for(int i=0; i<box[k].boxns; i++) {
		ff[k][i].x = 0.0;
		ff[k][i].y = 0.0;
		ff[k][i].z = 0.0;
	}
	/* ================================================================== */
	/*                                                                    */
	/* Call subroutines that calculate energy and forces from the         */
	/* different contributions to the force field.                        */
	/*                                                                    */
	/* ================================================================== */

	/* ---------------------------------------------------------- */
	/* GOLIK (not BGO)  Forcefield WITHOUT CONFIGT                */
	/* ---------------------------------------------------------- */
	double interen[12];

	en[k].bond = cbb_golik(k); //bond
	en[k].bend = cba_golik(k); //bend
	en[k].tors = cda_golik(k); //tors

	cnbnd_dnagolik(k,interen);
	en[k].nbond	= interen[0];
	en[k].nbonds	= interen[1];
	en[k].dna_stack = interen[2]; // enative
	en[k].dna_nonnat = interen[3]; // enonnative+emm
	en[k].dna_bp = interen[4]; // ebp
	en[k].coulomb = interen[5];
	en[k].dna_solv = interen[11];
}
