/* ===================================================== */
/* generate_topology.c                                   */
/* This subroutine generates the topology for the        */
/* system.                                               */
/* ===================================================== */

#include "defines.h"

void generate_topology(void){
	int i,ii,jj;
	int j;
	int j1 = 0; //bond number of strand 1
	int k1 = 0; //atom number of strand 1
	int j2; //bond number of strand 2
	int k2 = 3 * sequence.N -1; //atom number of strand 2
	int current_nick1, current_nick2;

	int bondN1, bondN2;
	/* Allocate memory for the bonds, bends, and torsions. */
	bondN1 = sequence.N*3-2;
	bondN2 = sequence.N*3-2;
	bondN_dna = bondN2 + bondN1; 

	j2 = bondN1;

	bond_dna = (struct bonds_dna*) calloc(bondN_dna,sizeof(struct bonds_dna));
	if(bond_dna == NULL){fprintf(stdout, "ERROR: cannot allocate memory for bond\n"); exit(11);}

	bendN_dna = bondN_dna * 3;  //just guess at the number of bends
	torsN_dna = bondN_dna * 3;  //just guess at the number of torsions
	bend_dna = (struct bends_dna*) calloc(bendN_dna,sizeof(struct bends_dna));
	if(bend_dna == NULL){fprintf(stdout, "ERROR: cannot allocate memory for bend\n"); exit(11);}
	tors_dna = (struct torss_dna*) calloc(torsN_dna,sizeof(struct torss_dna));
	if(tors_dna == NULL){fprintf(stdout, "ERROR: cannot allocate memory for tors\n"); exit(11);}

	/* Loop around the strands and connect the beads */
	/* j1 and k1 refer to the first strand and j2    */
	/* k2 refer to the second strand.                */
	current_nick1=-1;
	current_nick2=-1;

	for(i=0; i<sequence.N; i++){
		if(i == 0){
			/* The sugar/base bond */
			bond_dna[j1].a = k1;
			bond_dna[j1].b = k1+1;
			j1 = j1 + 1;

			bond_dna[j2].a = k2;
			bond_dna[j2].b = k2+1;
			j2 = j2 + 1;

			/* The 3'-sugar/phospate bond */
			if(current_nick1 != i){
				bond_dna[j1].a = k1;
				bond_dna[j1].b = k1+2;
				j1 = j1 + 1;
			}else{

			}
			if(current_nick2 != i){
				bond_dna[j2].a = k2;
				bond_dna[j2].b = k2+2;
				j2 = j2 + 1;
			}else{

			}

			k1 = k1 + 2;
			k2 = k2 + 2;
		}else{

			/* The phosphate/5'-sugar bond */
			bond_dna[j1].a = k1;
			bond_dna[j1].b = k1+1;
			j1 = j1 + 1;

			bond_dna[j2].a = k2;
			bond_dna[j2].b = k2+1;
			j2 = j2 + 1;

			/* The sugar/base bond */
			bond_dna[j1].a = k1+1;
			bond_dna[j1].b = k1+2;
			j1 = j1 + 1;

			bond_dna[j2].a = k2+1;
			bond_dna[j2].b = k2+2;
			j2 = j2 + 1;

			/* The 3'-sugar/phospate bond */

			if(i != sequence.N-1){
				if(current_nick1 != i){
					bond_dna[j1].a = k1+1;
					bond_dna[j1].b = k1+3;
					j1 = j1 + 1;
				}else{

				}
				if(current_nick2 != i){
					bond_dna[j2].a = k2+1;
					bond_dna[j2].b = k2+3;
					j2 = j2 + 1;
				}else{

				}
			}
			k1 = k1 + 3;
			k2 = k2 + 3;
		}

	}

	if(j2 != bondN_dna || j1!=bondN1){
		printf("There is a problem with the number of bonds\n");
		exit(20);
	}

	/* ----------------------------------------------- */
	/* Determine the bends from the bonds.             */
	/* ----------------------------------------------- */    
	j = 0;
	for(ii=0; ii<bondN_dna-1; ii++){
		for(jj=ii+1; jj<bondN_dna; jj++){
			if(bond_dna[ii].a == bond_dna[jj].a){
				bend_dna[j].a = bond_dna[ii].b;
				bend_dna[j].b = bond_dna[ii].a;
				bend_dna[j].c = bond_dna[jj].b;
				j++;
			}
			else if(bond_dna[ii].a == bond_dna[jj].b){
				bend_dna[j].a = bond_dna[ii].b;
				bend_dna[j].b = bond_dna[ii].a;
				bend_dna[j].c = bond_dna[jj].a;
				j++;
			}
			else if(bond_dna[ii].b == bond_dna[jj].a){
				bend_dna[j].a = bond_dna[ii].a;
				bend_dna[j].b = bond_dna[ii].b;
				bend_dna[j].c = bond_dna[jj].b;
				j++;
			}
			else if(bond_dna[ii].b == bond_dna[jj].b){
				bend_dna[j].a = bond_dna[ii].a;
				bend_dna[j].b = bond_dna[ii].b;
				bend_dna[j].c = bond_dna[jj].a;
				j++;
			}
		}//for jj
	}//for ii
	bendN_dna = j;

	/* ----------------------------------------------- */
	/* Determine the torsions from the bonds.          */
	/* ----------------------------------------------- */  
	j = 0;

	for(ii=0; ii<bendN_dna-1; ii++){
		for(jj=ii+1; jj<bendN_dna; jj++){
			if(bend_dna[ii].c == bend_dna[jj].b){
				if(bend_dna[ii].b == bend_dna[jj].a){
					tors_dna[j].a = bend_dna[ii].a;
					tors_dna[j].b = bend_dna[ii].b;
					tors_dna[j].c = bend_dna[ii].c;
					tors_dna[j].d = bend_dna[jj].c;
					j++;
				}else if(bend_dna[ii].b == bend_dna[jj].c){
					tors_dna[j].a = bend_dna[ii].a;
					tors_dna[j].b = bend_dna[ii].b;
					tors_dna[j].c = bend_dna[ii].c;
					tors_dna[j].d = bend_dna[jj].a;
					jj++;
				}
			}else if(bend_dna[ii].a == bend_dna[jj].b){
				if(bend_dna[ii].b == bend_dna[jj].c){
					tors_dna[j].a = bend_dna[jj].a;
					tors_dna[j].b = bend_dna[jj].b;
					tors_dna[j].c = bend_dna[jj].c;
					tors_dna[j].d = bend_dna[ii].c;
					jj++;
				}else if(bend_dna[ii].b == bend_dna[jj].a){
					tors_dna[j].a = bend_dna[jj].c;
					tors_dna[j].b = bend_dna[jj].b;
					tors_dna[j].c = bend_dna[jj].a;
					tors_dna[j].d = bend_dna[ii].c;
					jj++;
				}
			}
		}//for jj
	}//for ii
	torsN_dna = j;
}

