/* ======================================================================== */
/* nblist.cpp                                                               */
/*                                                                          */
/*		This subroutine sets up the neighbor-list for each molecule. It is  */
/* called in main only if NLIST is defined.                                 */
/*                                                                          */
/*                                                                          */
/* Passed Parameters:                                                       */
/*						ibox:		The box number                          */
/*                                                                          */
/* ======================================================================== */

#include "defines.h"

/* ************************************************************************ */
/* ======================== Begin Subroutine ============================== */
/* ************************************************************************ */
void nblist (int ibox)
{
	int k = ibox;

	double rcut2 = sim.rclist * sim.rclist;

	int ns = box[k].boxns;

	/* ================================================================== */
	/* Zero out count accumulators.                                       */
	/* ================================================================== */
	for(int i=0; i<ns; i++)
	{
		nlist[k].count[i] = 0;
	}
	/* ================================================================== */
	/* Loops to create neighbor list.                                     */
	/* ================================================================== */
	for(int a=0; a<ns-1; a++)
	{

		/* ----------------------------------------------- */
		/* Assign position of particle a.                  */
		/* ----------------------------------------------- */    
		double ax = atom[k][a].x;
		double ay = atom[k][a].y;
		double az = atom[k][a].z;

		/* ----------------------------------------------- */
		/* Loop around atoms "above" particle a in array.  */
		/* ----------------------------------------------- */
		for(int b=a+1; b<ns; b++)
		{
			/* ----------------------------------------------- */
			/* Calculate the x,y,z distances between           */
			/* particles a & b and apply minimum image.        */
			/* ----------------------------------------------- */

			double dx = ax - atom[k][b].x;
			double dy = ay - atom[k][b].y;
			double dz = az - atom[k][b].z;

			/* ----------------------------------------------- */
			/* Calculate the distance between particles a & b. */
			/* If the distance is less than the cutoff,        */
			/* add the particle to the list. If it is greater, */
			/* go to the next atom in the loop.                */
			/* ----------------------------------------------- */
			double dr2 = dx*dx + dy*dy + dz*dz;

			if(dr2 < rcut2)
			{   
				nlist[k].list[a][nlist[k].count[a]] = b;
				nlist[k].xflag[a][nlist[k].count[a]] = 0;  //the flag is zero if the pair is not part of a bond, bend, or torsion
				/* ----------------------------------------------- */
				/* Search through the list of excluded             */
				/* interactions and assign the appropriate flag    */
				/* to the neighboring pair.                        */
				/* ----------------------------------------------- */
				if(xintN[k])
				{
					int index = (a*b) % NPRIME;
					int n = table[k][index].n;
					for(int r=0; r<n; r++)
					{
						int ra = table[k][index].a[r];
						int rb = table[k][index].b[r];
						if(((a == ra) && (b == rb)) || ((a == rb) && (b == ra)))
						{
							nlist[k].xflag[a][nlist[k].count[a]] = xint[k][table[k][index].p[r]].xflag;
							nlist[k].sig[a][nlist[k].count[a]]=xint[k][table[k][index].p[r]].sig;
						}//if(((a == ra) && (b == rb)) || ((a == rb) && (b == ra)))
					}//for r
				}// xintN[k]
				nlist[k].count[a]++;
				if(nlist[k].count[a] > max_num_neighbors)
				{
					printf("The neighborlist array is too small.  Increase max_num_neighbors to at least %i in allocate.cpp or check your density!\n", nlist[k].count[a]);
					exit(5);
				}

			}// if(dr2 < rcut2)
		}//end b loop
	}// end a loop
}

