/* ======================================================================== */
/* read_topology.cpp                                                        */
/*                                                                          */
/*		This subroutine reads in the molecular topology for all molecules.  */
/* The topology contains the information telling the atoms involved in each */
/* bond, bend, torsion, and improper dihedral that the system contains.     */
/* The file, named simul#.psf, comes from a CHARMM output.                  */
/*                                                                          */  
/* Note: There is a separate simul#.psf file for each box.                  */
/*                                                                          */
/* ======================================================================== */

#include "defines.h"
int duplicate14_check(int, int,int, int*, int*, int);
/* ************************************************************************ */
/*                                                                          */
/* ======================== Begin Subroutine ============================== */
/*                                                                          */
/* ************************************************************************ */
void read_topology (int ibox)
{

	/* ================================================================== */
	/*                                                                    */
	/* Variables and pointers needed in the subroutine.                   */
	/*                                                                    */
	/* ================================================================== */
	char amino_name[47][4]={"ABC","GLY","ALA","SER","CYS","VAL","THR","ILE","PRO","MET","ASP","ASN","LEU","LYS","GLU",
		"GLN","ARG","HIS","PHE","TYR","TRP","CYT","GUA","ADE","THY","HOH","SOD","CLA","LIP","CHO","EAM","POT","TRE", "GOL","F00", "F01",
		"F02","F03","F04","F05","F06","F07","F08","F09","F10","F11","MEO"};
	int k = ibox;
	int t1; int flag=0; int rescount=0; int temp;
	char am[5];

	int Nsite=0;
	int Nmolec = -1;
	int sites;
	for (int cc=0; cc<sim.NC; cc++){

		/* ----------------------------------------------- */
		/* Read in the information about each atom/sites.  */
		/* ----------------------------------------------- */    
		sites = natom_dna.sod/2;
		if(sites!= (mol[cc].Nsite*bp[k][cc].nbox)){
			fprintf(stdout,"Number of total sites in crd (%i) and psf (%i) for molecule type %d  do not match!!! \n", mol[cc].Nsite*bp[k][cc].nbox, sites, cc);
			exit(102);
		}
		for (int i=0; i<(mol[cc].Nsite*bp[k][cc].nbox); i++){
			if(mol[cc].Nsite == 1) Nmolec++;
			else if(i%(mol[cc].Nsite+1)==0) Nmolec++;
			temp = i+1;
			rescount = atom_dna[i].residue+1;

			if(rescount > box[k].boxnres)
			{       
				fprintf(stdout, "Atom %i of molecule %i in simul.psf has residue number %i, but total number of residues in box %i in mol.input is %i !!!\n", i, cc, rescount, k, box[k].boxnres);
				exit(102);
			} 

			atom[k][Nsite].residue=rescount-1;			// subtract off one to make first residue 0
			atom[k][Nsite].molec = Nmolec;
			residue[k][rescount-1].Nsite++;
			residue[k][rescount-1].chain=cc;	// setting the chain to the molecule type
			strcpy(am, atom_dna[i].res_name);
			/* *********************************************** */
			/* Compare the amino acid name to list above to    */
			/* assign the appropriate number code in .type.    */
			/* *********************************************** */
			for(int ii=1;ii<47;ii++){
				if(strcmp(amino_name[ii],am)==0)
					residue[k][rescount-1].type=ii;
			}
			/* *********************************************** */
			/* Read in the site/atom name and id number.       */
			/* *********************************************** */
			strcpy(atom[k][Nsite].site_name, atom_dna[Nsite].site_name);
			atom[k][Nsite].site_id = atom_dna[Nsite].site_id;
			for (int kk =0; kk<NUM_ATOM; kk++) {
				/* *********************************************** */
				/* Compare the id number read to those read in     */
				/* read_atom.  If the same, copy the name to       */
				/* atnopbc[k][i].t.  Note: This name is more       */
				/* descriptive than those read in read_atom.       */
				/* *********************************************** */
				if (atom[k][Nsite].site_id== atom_type[kk].atomid){
					strcpy(atom[k][Nsite].t,atom_type[kk].name);
					break;
				}
			}

			/* *********************************************** */
			/* Read in charge.                                 */
			/* *********************************************** */
			atom[k][Nsite].charge = atom_dna[Nsite].charge;
			Nsite++;

		}//for i
	}//for cc

	/* *********************************************** */
	/* Read in the number of bonds.                    */
	/* *********************************************** */
	bondN[k] = bondN_dna;
	/* *********************************************** */
	/* Allocate memory for this number of bonds.       */
	/* *********************************************** */
	int rb = bondN[k] + MEXTRA;
	bond[k] = (struct bonds*) calloc(rb,sizeof(struct bonds));
	if(bond[k] == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for bond[k]\n"); exit(11);}
	/* *********************************************** */
	/* Variables needed.                               */
	/* *********************************************** */
	int count=0; t1=0;
	for(int i=0; i<bondN[k]; i++) {

		/* *********************************************** */
		/* Read in the two atoms/site numbers involved     */
		/* in each bond. Subtract 1 to make the site       */
		/* numbers begin at 0.                             */
		/* *********************************************** */
		bond[k][i].a = bond_dna[i].a;
		bond[k][i].b = bond_dna[i].b;
		count++;
		for (int nb =0; nb<NUM_BOND; nb++) {
			/* *********************************************** */
			/* Compare the site id numbers(type of site) to    */
			/* the list read in read_atom.  When they match,   */
			/* assign that bond the appropriate req and        */
			/* krbond.  If no matches are found, there is an   */
			/* error in the input files so exit.               */
			/* *********************************************** */
			if ((bond_prop[nb].type1 == atom[k][bond[k][i].a].site_id && bond_prop[nb].type2 == atom[k][bond[k][i].b].site_id) || 
					(bond_prop[nb].type1 == atom[k][bond[k][i].b].site_id && bond_prop[nb].type2 == atom[k][bond[k][i].a].site_id)) {
				bond[k][i].req    = bond_prop[nb].eqbond;
				bond[k][i].krbond = bond_prop[nb].kbond;
				flag=1;break; 
			}
		}
		if (flag==0) {
			fprintf(stdout,"ERROR: bond parameters not defined between atoms %i and %i !!!\n", bond[k][i].a+1, bond[k][i].b+1);
			exit(1);
		}

		/* *********************************************** */
		/* Convert from kcal/mol to kJ/mol.                */
		/* *********************************************** */
		bond[k][i].krbond  *= 4.184;
		flag=0;
	} 

	/* *********************************************** */
	/* Read in the number of bends.                    */
	/* *********************************************** */
	bendN[k] = bendN_dna;
	/* *********************************************** */
	/* Allocate memory for this number of bends.       */
	/* *********************************************** */
	int rc = bendN[k] + MEXTRA;
	bend[k] = (struct bends*) calloc(rc,sizeof(struct bends));
	if(bend[k] == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for bend[i]\n"); exit(11);}
	count =0; t1=0;
	/* *********************************************** */
	/* Variables needed.                               */
	/* *********************************************** */
	for(int i=0; i<bendN[k]; i++) {

		/* *********************************************** */
		/* Read in the three atoms/site numbers involved   */
		/* in each bend. Subtract 1 to make the site       */
		/* numbers begin at 0.                             */
		/* *********************************************** */
		bend[k][i].a = bend_dna[i].a;
		bend[k][i].b = bend_dna[i].b;
		bend[k][i].c = bend_dna[i].c;
		count++;
		/* *********************************************** */
		/* Compare the site id numbers(type of site) to    */
		/* the list read in read_atom.  When they match,   */
		/* assign that bend the appropriate eqbend and     */
		/* kbend.  If no matches are found, there is an   */
		/* error in the input files so exit.               */
		/* *********************************************** */
		for (int nb =0; nb< NUM_BEND; nb++) {
			if((bend_prop[nb].type1==atom[k][bend[k][i].a].site_id && bend_prop[nb].type2==atom[k][bend[k][i].b].site_id 
						&& bend_prop[nb].type3==atom[k][bend[k][i].c].site_id) ||  (bend_prop[nb].type1==atom[k][bend[k][i].c].site_id 
							&& bend_prop[nb].type2==atom[k][bend[k][i].b].site_id && bend_prop[nb].type3==atom[k][bend[k][i].a].site_id)) {
				bend[k][i].angeq  = bend_prop[nb].eqbend;
				bend[k][i].krbend = bend_prop[nb].kbend;
				flag=1;break; 
			}
		}
		if (flag==0) {
			fprintf(stdout,"ERROR: bend parameter not defined between atoms %d, %d, %d !!!\n",bend[k][i].a+1,bend[k][i].b+1,bend[k][i].c+1);
			exit(1);
		}

		/* *********************************************** */
		/* Convert from kcal/mol to kJ/mol and degrees to  */
		/* radians.                                        */
		/* *********************************************** */
		bend[k][i].krbend *= 4.184;
		bend[k][i].angeq  *= PI/180.0;flag=0;
	}

	/* *********************************************** */
	/* Read in the number of torsions.                 */
	/* *********************************************** */
	torsN[k] = torsN_dna;
	/* *********************************************** */
	/* Variables needed.                               */
	/* *********************************************** */
	int rd = torsN[k] + MEXTRA;
	/* *********************************************** */
	/* Allocate memory for this number of torsions.    */
	/* *********************************************** */
	tors[k] = (struct torsions*) calloc(rd,sizeof(struct torsions));
	if(tors[k] == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for tors[k]\n"); exit(11);}
	count=0; t1=0;
	for(int i=0; i<torsN[k]; i++) {
		/* *********************************************** */
		/* Read in the four atoms/site numbers involved    */
		/* in each bend. Subtract 1 to make the site       */
		/* numbers begin at 0.                             */
		/* *********************************************** */
		tors[k][i].a = tors_dna[k].a;
		tors[k][i].b = tors_dna[k].b;
		tors[k][i].c = tors_dna[k].c;
		tors[k][i].d = tors_dna[k].d;

		count++;int tt=0;
		for (int nb=0; nb< NUM_TORS; nb++) {
			/* *********************************************** */
			/* Compare the site id numbers(type of site) to    */
			/* the list read in read_atom.  When they match,   */
			/* assign that torsion the appropriate kphi,nphi   */
			/* and delphi. If no matches are found, there is   */
			/* an error in the input files so exit.            */
			/* *********************************************** */

			if ((dihedral_prop[nb].type1==atom[k][tors[k][i].a].site_id && dihedral_prop[nb].type2==atom[k][tors[k][i].b].site_id 
						&&dihedral_prop[nb].type3==atom[k][tors[k][i].c].site_id  && dihedral_prop[nb].type4==atom[k][tors[k][i].d].site_id)
					|| (dihedral_prop[nb].type1==atom[k][tors[k][i].d].site_id && dihedral_prop[nb].type2==atom[k][tors[k][i].c].site_id 
						&&dihedral_prop[nb].type3==atom[k][tors[k][i].b].site_id  && dihedral_prop[nb].type4==atom[k][tors[k][i].a].site_id)) {

				tors[k][i].kphi[tt]= 4.184*dihedral_prop[nb].kphi;
				tors[k][i].nphi[tt]= dihedral_prop[nb].nphi;
				tors[k][i].delphi[tt]= PI*dihedral_prop[nb].delphi/180; tt++;
				flag=1;//break;
			}
			else if ((( dihedral_prop[nb].type1==100 && dihedral_prop[nb].type4 ==100 && dihedral_prop[nb].type2==atom[k][tors[k][i].b].site_id 
							&& dihedral_prop[nb].type3==atom[k][tors[k][i].c].site_id) || (dihedral_prop[nb].type1==100 && dihedral_prop[nb].type4 ==100 
								&& dihedral_prop[nb].type2==atom[k][tors[k][i].c].site_id &&dihedral_prop[nb].type3==atom[k][tors[k][i].b].site_id)) && flag==0) {
				tors[k][i].kphi[0]= 4.184*dihedral_prop[nb].kphi;
				tors[k][i].nphi[0]= dihedral_prop[nb].nphi;
				tors[k][i].delphi[0]= PI*dihedral_prop[nb].delphi/180;
				flag=1;break; 
			}
		}
		if (flag==0) {
			fprintf(stdout,"ERROR: dihedral parameters not defined for atom types	%d	%d	%d	%d!!!\n",atom[k][tors[k][i].a].site_id,
					atom[k][tors[k][i].b].site_id,atom[k][tors[k][i].c].site_id,atom[k][tors[k][i].d].site_id);
			exit(1);
		}
		flag=0;
	}

	/* ------------------------------------------------		*/
	/*	CHARMM 19 Convention					*/
	/* Make a list of the main chain phi and psi angles		*/
	/* Phi defined as: C(i-1) - N(i) - CA(i) - N(i+1)		*/
	/* Psi defined as:	N(i) - CA(i) - C(i) - N(i+1)		*/
	/*								*/
	/*	CA	=	12 if CH1E	alpha carbon		*/ 
	/*		=	13 if CH2E	alpha carbon		*/
	/*	N	=	31 (N)/34 (NH1E)/35 (NH2E)/36 (NH3E)	*/
	/*		=	38 (NH1)/39 (NH2)/40 (NH3)		*/
	/*	C	=	11	for carbonyl carbon C=O		*/
	/* ------------------------------------------------		*/


	/* ------------------------------------------------		*/
	/*	CHARMM 22 Convention					*/
	/* Make a list of the main chain phi and psi angles		*/
	/* Phi defined as: C(i-1) - N(i) - CA(i) - N(i+1)		*/
	/* Psi defined as:	N(i) - CA(i) - C(i) - N(i+1)		*/
	/*								*/
	/*	CA	=	22 if CH1E	alpha carbon		*/ 
	/*		=	23 if CH2E	alpha carbon		*/
	/*		=	29 if Proline ring carbon		*/
	/*	N	=	50 (Proline N)/54 (NH1)/55 (NH2)	*/
	/*		=	56 (NH3)/59 (terminal proline NH2+)	*/
	/*	C	=	20,32,33  for carbonyl carbon C=O	*/
	/* ------------------------------------------------		*/

	for(int i=0; i<torsN[k]; i++) {
		if ((atom[k][tors[k][i].a].site_id == 20 || atom[k][tors[k][i].a].site_id == 32 ||
					atom[k][tors[k][i].a].site_id == 33)&&(atom[k][tors[k][i].b].site_id == 50 ||
						atom[k][tors[k][i].b].site_id == 54)&&
				(atom[k][tors[k][i].c].site_id == 22 || atom[k][tors[k][i].c].site_id == 23 ||
				 atom[k][tors[k][i].c].site_id == 29)&&(atom[k][tors[k][i].d].site_id == 20 ||
					 atom[k][tors[k][i].d].site_id == 32 || atom[k][tors[k][i].d].site_id == 33)){
			tors[k][i].phitag=1;
			tors[k][i].psitag=0;
			tors[k][i].thetag=0;
		}
		else if((atom[k][tors[k][i].a].site_id == 50 || atom[k][tors[k][i].a].site_id == 54 ||
					atom[k][tors[k][i].a].site_id == 55 || atom[k][tors[k][i].a].site_id == 56 ||
					atom[k][tors[k][i].a].site_id == 59)&&(atom[k][tors[k][i].b].site_id == 22 || 
						atom[k][tors[k][i].b].site_id == 23 || atom[k][tors[k][i].b].site_id == 29)&&
				(atom[k][tors[k][i].c].site_id == 20 || atom[k][tors[k][i].c].site_id == 32 || 
				 atom[k][tors[k][i].c].site_id == 33)&&(atom[k][tors[k][i].d].site_id == 50||
					 atom[k][tors[k][i].d].site_id == 54)){
			tors[k][i].phitag=0;
			tors[k][i].psitag=1;
			tors[k][i].thetag=0;
		}
		else{
			tors[k][i].phitag=0;
			tors[k][i].psitag=0;
			tors[k][i].thetag=0;
		}
	}

	/* ----------------------------------------------- */
	/* Assign the Lennard Jones parameters to each     */
	/* atom/site.  The parameters were read in         */
	/* read atom into the epstmp and sigtmp arrays.    */
	/* ----------------------------------------------- */    
	/* *********************************************** */
	/* Assign memory for LJ paramters.                 */
	/* *********************************************** */
	int ra = box[k].boxns + MEXTRA;
	pott[k] = (struct inter*) calloc(ra,sizeof(struct inter));
	if(pott[k] == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for pott[k]\n"); exit(11);}

	for(int i=0; i<box[k].boxns; i++) {
		/* *********************************************** */
		/* Assign the potential id numbers to each site i. */
		/* *********************************************** */

		for (int mm =0; mm<NUM_ATOM;mm++){
			/* *********************************************** */
			/* Compare the id number to those read in          */
			/* read_atom (atom_type array). If they are the    */
			/* same, assign the correct mass to the site.      */
			/* *********************************************** */
			if(atom_type[mm].atomid == atom[k][i].site_id){
				pott[k][i].mas = atom_type[mm].mass; 

				break;
			}
		}

		/* *********************************************** */
		/* Assign the correct eps, sig, and qq to each     */
		/* site.  Note: The index of epstmp and sigtmp are */
		/* over id numbers, not site numbers.              */
		/* *********************************************** */
		pott[k][i].eps  = -4.184* epstmp[0][atom[k][i].site_id];
		pott[k][i].sig  = 2*sigtmp[0][atom[k][i].site_id];
		pott[k][i].qq   = atom[k][i].charge;

	}

	/* *********************************************** */
	/* Allocate memory for the ljset structure to be   */
	/* used in setlj().                                */
	/* *********************************************** */

	int rf = NUM_LJSET;   
	ljset = (struct inter**) calloc(rf,sizeof(struct inter*));
	if(ljset == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for ljset\n"); exit(11);}

	for(int n=0; n<rf; n++){ 
		ljset[n] = (struct inter*) calloc(rf,sizeof(struct inter));
		if(ljset[n] == NULL){ fprintf(stdout, "ERROR: cannot allocate memory for ljset[n]\n"); exit(11);}

	}

}

/* ======================================================================== */
/* duplicate14_check.cpp                                                    */
/*                                                                          */
/*		This subroutine determines if the 1-4 interaction found above by    */
/* searching through the bends is already part of a bond or bend or if it   */
/* was specifically exluded as a fixed NB exclusion.  All these usually     */
/* happen in ring structures.  If the 1-4 interaction in question is one of */
/* the above elements, the subroutine returns 1, else, it returns 0.        */
/*                                                                          */
/* ======================================================================== */

/* ************************************************************************ */
/*                                                                          */
/* ======================== Begin Subroutine ============================== */
/*                                                                          */
/* ************************************************************************ */
int duplicate14_check(int ibox, int a, int d, int *a14, int *d14, int count )
{
	int k=ibox;
	/* ----------------------------------------------- */
	/* Loop over the bonds.                            */
	/* ----------------------------------------------- */    
	for(int i=0; i<bondN[k]; i++){
		if( (bond[k][i].a == a && bond[k][i].b == d)||
				(bond[k][i].b == a && bond[k][i].a == d))
			return(1);
	}

	/* ----------------------------------------------- */
	/* Loop over the bends.                            */
	/* ----------------------------------------------- */   
	for(int i=0; i<bendN[k]; i++){
		if( (bend[k][i].a == a && bend[k][i].c == d)||
				(bend[k][i].c == a && bend[k][i].a == d))
			return(1);
	}

	/* ----------------------------------------------- */
	/* Remove any double counts for 1-4	             */
	/* ----------------------------------------------- */   
	for(int i=0; i<count; i++){
		if( (a14[i] == a && d14[i] == d)||
				(d14[i] == a && a14[i] == d))
			return(1);
	}
	return(0);

}
